// --------------------------------------------------------------------------------------
//
// BasisKlassen.CS                                                 Ulrich Mller 23.01.05
//                                                                 PCS1700       C# Final
// Basisklassen fr die Graphiken
//
// --------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Threading;

namespace DivGraph
{
	//
	// --- UrGraf : Basis fr Graphics-Graphik (mit Draw ...) ----------
	//              ausgabe dient der Angabe der Gre der Zeichenflche
	//              das eigentliche Zeichenobjekt ist canvas.
	public abstract class UrGraf	{
		protected Control  ausgabe;
		protected Graphics canvas;
		protected Pen      stift;
		protected Color    background;

		public UrGraf(Control Ausgabe)	{
			this.ausgabe    = Ausgabe;
			this.stift      = new Pen(Color.Black, 1);
			this.background = Color.White;
		}
		public Pen Stift {
			get {return stift;}
			set {stift = value;}
		}
		public Color Background {
			get {return background;}
			set {background = value;}
		}
		public virtual void Action() {
			Action(ausgabe.CreateGraphics());
		}
		public abstract void Action(Graphics Canvas);
	}

	// --- Turtle : Logo Turtle-Graphik -----------------------------------------
	//     Koordinaten 0,0 Mitte Zeichenflche, Einheiten Pixel
	//     PenColor : Black, 1; Background : White
	//     Befehlsumfang nach H.Abelson Einfhrung in Logo, Seit 141 ff.
	//                                  deutsch / englisch
	//
	public class Turtle {
		private const float WR = 180F / (float)Math.PI;

		protected Control  ausgabe;
		protected Graphics canvas;
		protected Pen      stift;
		protected Color    background;
		private   bool     justStarted;

		private bool  stiftState;
		private float currentX;
		private float currentY;
		private float currentDir;

		private Random rnd;

		public Turtle(Control Ausgabe) {
			this.ausgabe     = Ausgabe;
			this.stift       = new Pen(Color.Black, 1);
			this.background  = Color.White;
			this.justStarted = true;
		}
		
		public void Draw(Graphics Canvas) {Bild(Canvas);}
		public void Draw(Graphics Canvas, Color CName) {Bild(Canvas, CName);}
		public void Bild(Graphics Canvas) {Bild(Canvas, this.background);}
		public void Bild(Graphics Canvas, Color CName) {
			this.canvas = Canvas;
			this.canvas.Clear(CName);
			if(justStarted) {
				this.canvas.TranslateTransform(this.ausgabe.Width/2, 
					                           this.ausgabe.Height/2);
				justStarted = false;
			}
			this.stiftState = false;
			this.currentX = 0;
			this.currentY = 0;
			this.currentDir = 0;
		}
		public void ClearScreen() {LoescheBild();}
		public void LoescheBild() {
			this.canvas.Clear(this.background);
		}
		public void PenColor(Color CName) {Farbe(CName);}
		public void Farbe(Color CName) {
			stift = new Pen(CName, 1);
		}
		public void PenDown() {StiftAb();}
		public void StiftAb() {
			stiftState = true;
		}
		public void PenUp() {StiftHoch();}
		public void StiftHoch() {
			stiftState = false;
		}
		public void Background(Color Background) {Hintergrund(Background);}
		public void Hintergrund(Color Farbe) {
			this.background = Farbe;
		}
		public void SetHeading(float Angle) {AufKurs (Angle);}
		public void AufKurs(float Winkel) {
			currentDir = Winkel;
		}
		public void SetX(float CoorX) {AufX(CoorX);}
		public void AufX(float KoorX) {
			if(stiftState) canvas.DrawLine(stift, currentX, currentY,
							  KoorX, currentY);
			currentX = KoorX;
		}
		public float XCor() {return XKo();}
		public float XKo() {
			return currentX;
		}
		public void SetY(float CoorY) {AufY(CoorY);}
		public void AufY(float KoorY) {
			if(stiftState) canvas.DrawLine(stift, currentX, currentY,
							  currentX, KoorY);
			currentY = KoorY;
		}
		public float YCor() {return YKo();}
		public float YKo() {
			return currentY;
		}
		public void SetXY(float CoorX, float CoorY) {AufXY(CoorX, CoorY);}
		public void AufXY(float KoorX, float KoorY) {
			if(stiftState) canvas.DrawLine(stift, currentX, currentY, KoorX, KoorY);
			currentX = KoorX;
			currentY = KoorY;
		}

		public float Heading() {return Kurs();}
		public float Kurs() {
			return currentDir * WR;
		}
		public float Towards(float CoorX, float CoorY) {return Richtung(CoorX, CoorY);}
		public float Richtung(float KoorX, float KoorY) {
			return (float)(Math.Atan(currentY - KoorY / currentX - KoorX));
		}
		public void Home() {Mitte();}
		public void Mitte() {
			currentX = 0;
			currentY = 0;
		}
		public void Forward(float Delta) {Vorwaerts(Delta);}
		public void Vorwaerts(float Delta) {
			float x2 = currentX + (float)Math.Sin(currentDir) * Delta;
			float y2 = currentY + (float)Math.Cos(currentDir) * Delta;
			if(stiftState) canvas.DrawLine(stift, currentX, currentY, x2, y2);
			currentX = x2;
			currentY = y2;
		}
		public void Back(float DeltaPixel) {Rueckwaerts(DeltaPixel);}
		public void Rueckwaerts(float DeltaPixel) {
			float x2 = currentX - (float)Math.Sin(currentDir) * DeltaPixel;
			float y2 = currentY - (float)Math.Cos(currentDir) * DeltaPixel;
			if(stiftState) canvas.DrawLine(stift, currentX, currentY, x2, y2);
			currentX = x2;
			currentY = y2;
		}
		public void Left(float DeltaDegree) {Links(DeltaDegree);}
		public void Links(float DeltaGrad) {
			currentDir -= DeltaGrad / WR;
		}
		public void Right(float DeltaDegree) {Rechts(DeltaDegree);}
		public void Rechts(float DeltaGrad) {
			currentDir += DeltaGrad / WR;
		}
		public void Randomize() {StarteZufall();}
		public void StarteZufall() {
			rnd = new Random();
		}
		public void Randomize(int Seed) {StarteZufall(Seed);}
		public void StarteZufall(int Seed) {
			rnd = new System.Random(Seed);
		}
		public int Random(int Ran) {return Zufallszahl(Ran);}
		public int Zufallszahl(int Zuf) {
			return rnd.Next(0, Zuf-1);
		}
		public void Pause(int ZSek) {
			Thread.Sleep(ZSek*100);
		}
	}
}
